/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
extern "C" {
#include "private.h"
}
#include "imgsettingswidget.h"
#include <QAction>

ImageSettingsWidget::ImageSettingsWidget(GenomeVector* gen, QWidget* parent)
	: QWidget(parent), genome(gen)
{
	setupUi(this);

	connect(m_qualityLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_filterLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_oversampleLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_batchesLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_temporalLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorCurveLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_estimatorMinLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_symmetryLineEdit, SIGNAL(valueUpdated()),
            this, SLOT(fieldEditedAction()));
	connect(m_filterShapeBox, SIGNAL(activated(int)),
            this, SLOT(fieldEditedAction()));
}

void ImageSettingsWidget::reset()
{
	updateFormData();
}

#define genome_ptr (genome->selectedGenome())

void ImageSettingsWidget::updateFormData()
{
	m_qualityLineEdit->updateValue(genome_ptr->sample_density);
	m_filterLineEdit->updateValue(genome_ptr->spatial_filter_radius);
	m_oversampleLineEdit->updateValue(genome_ptr->spatial_oversample);
	m_batchesLineEdit->updateValue(genome_ptr->nbatches);
	m_temporalLineEdit->updateValue(genome_ptr->ntemporal_samples);
	m_estimatorLineEdit->updateValue(genome_ptr->estimator);
	m_estimatorCurveLineEdit->updateValue(genome_ptr->estimator_curve);
	m_estimatorMinLineEdit->updateValue(genome_ptr->estimator_minimum);
	m_symmetryLineEdit->updateValue(genome_ptr->symmetry);

	// filter_shape
	QString fshape = "gaussian";
	if (genome_ptr->spatial_filter_func == gaussian_filter)
		fshape ="gaussian";
	else if (genome_ptr->spatial_filter_func == hermite_filter)
		fshape ="hermite";
	else if (genome_ptr->spatial_filter_func == box_filter)
		fshape ="box";
	else if (genome_ptr->spatial_filter_func == triangle_filter)
		fshape ="triangle";
	else if (genome_ptr->spatial_filter_func == bell_filter)
		fshape ="bell";
	else if (genome_ptr->spatial_filter_func == b_spline_filter)
		fshape ="bspline";
	else if (genome_ptr->spatial_filter_func == mitchell_filter)
		fshape ="mitchell";
	else if (genome_ptr->spatial_filter_func == blackman_filter)
		fshape ="blackman";
	else if (genome_ptr->spatial_filter_func == catrom_filter)
		fshape ="catrom";
	else if (genome_ptr->spatial_filter_func == hanning_filter)
		fshape ="hanning";
	else if (genome_ptr->spatial_filter_func == hamming_filter)
		fshape ="hamming";
	else if (genome_ptr->spatial_filter_func == lanczos3_filter)
		fshape ="lanczos3";
	else if (genome_ptr->spatial_filter_func == lanczos2_filter)
		fshape ="lanczos2";
	else if (genome_ptr->spatial_filter_func == quadratic_filter)
		fshape ="quadratic";
	m_filterShapeBox->setCurrentIndex(m_filterShapeBox->findText(fshape));
}

void ImageSettingsWidget::fieldEditedAction()
{
	genome_ptr->sample_density = qMax(0.00001, m_qualityLineEdit->value());
	genome_ptr->spatial_filter_radius = m_filterLineEdit->value();
	genome_ptr->spatial_oversample = m_oversampleLineEdit->value();
	genome_ptr->nbatches = m_batchesLineEdit->value();
	genome_ptr->ntemporal_samples = QString(m_temporalLineEdit->text()).toInt();
	genome_ptr->estimator = m_estimatorLineEdit->value();
	genome_ptr->estimator_curve = m_estimatorCurveLineEdit->value();
	genome_ptr->estimator_minimum = m_estimatorMinLineEdit->value();
	genome_ptr->symmetry = m_symmetryLineEdit->value();

	QString fshape = m_filterShapeBox->currentText();
	if (fshape == "gaussian") {
		genome_ptr->spatial_filter_func = &gaussian_filter;
		genome_ptr->spatial_filter_support = gaussian_support;
	} else if (fshape == "hermite") {
		genome_ptr->spatial_filter_func = hermite_filter;
		genome_ptr->spatial_filter_support = hermite_support;
	} else if (fshape == "box") {
		genome_ptr->spatial_filter_func = box_filter;
		genome_ptr->spatial_filter_support = box_support;
	} else if (fshape == "triangle") {
		genome_ptr->spatial_filter_func = triangle_filter;
		genome_ptr->spatial_filter_support = triangle_support;
	} else if (fshape == "bell") {
		genome_ptr->spatial_filter_func = bell_filter;
		genome_ptr->spatial_filter_support = bell_support;
	} else if (fshape == "bspline") {
		genome_ptr->spatial_filter_func = b_spline_filter;
		genome_ptr->spatial_filter_support = b_spline_support;
	} else if (fshape == "mitchell") {
		genome_ptr->spatial_filter_func = mitchell_filter;
		genome_ptr->spatial_filter_support = mitchell_support;
	} else if (fshape == "blackman") {
		genome_ptr->spatial_filter_func = blackman_filter;
		genome_ptr->spatial_filter_support = blackman_support;
	} else if (fshape == "catrom") {
		genome_ptr->spatial_filter_func = catrom_filter;
		genome_ptr->spatial_filter_support = catrom_support;
	} else if (fshape == "hanning") {
		genome_ptr->spatial_filter_func = hanning_filter;
		genome_ptr->spatial_filter_support = hanning_support;
	} else if (fshape == "hamming") {
		genome_ptr->spatial_filter_func = hamming_filter;
		genome_ptr->spatial_filter_support = hamming_support;
	} else if (fshape == "lanczos3") {
		genome_ptr->spatial_filter_func = lanczos3_filter;
		genome_ptr->spatial_filter_support = lanczos3_support;
	} else if (fshape == "lanczos2") {
		genome_ptr->spatial_filter_func = lanczos2_filter;
		genome_ptr->spatial_filter_support = lanczos2_support;
	} else if (fshape == "quadratic") {
		genome_ptr->spatial_filter_func = quadratic_filter;
		genome_ptr->spatial_filter_support = quadratic_support;
	} else {
		genome_ptr->spatial_filter_func = gaussian_filter;
		genome_ptr->spatial_filter_support = gaussian_support;
	}

	emit dataChanged();
}


