/***************************************************************************
 *   Copyright (C) 2007 by David Bitseff                                   *
 *   dbitsef@zipcon.net                                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "logger.h"
#include "luathread.h"
#include "genome.h"
#include "xform.h"

#include "mainwindowadapter.h"

namespace Lua
{
LuaThread::LuaThread(MainWindow* m, QObject* parent)
	: mw(m)
{
	f = 0;
}

LuaThread::~LuaThread()
{
}


void LuaThread::run()
{
	lua_State *L = luaL_newstate();
	luaL_openlibs(L);

	Lunar<Frame>::Register(L);
	Lunar<Genome>::Register(L);
	Lunar<XForm>::Register(L);

	luaL_getmetatable(L, Frame::className);
	Lunar<Frame>::new_T(L);
	f = Lunar<Frame>::check(L,1);
	MainWindowAdapter ctx(mw, this);
	f->setMainWindow(&ctx);
	lua_setglobal(L, "frame");
	lua_load_environment(L);
	mw->setDialogsEnabled(false);
	int error = luaL_loadstring(L, lua_text.toAscii().constData())
					|| lua_pcall(L, 0, LUA_MULTRET, 0);
	if (error)
	{
		logError(QString("ScriptEditWidget::runScriptAction : %1")
				.arg(lua_tostring(L, -1)));
		lua_pop(L, 1);  /* pop error message from the stack */
	}

	lua_close(L);
	f = 0;
	mw->setDialogsEnabled(true);
	emit scriptFinished();
}

void LuaThread::msleep(unsigned long msecs)
{
	QThread::msleep(msecs);
}

void LuaThread::lua_load_environment(lua_State* L)
{
	lua_pushinteger(L, VAR_LINEAR); lua_setglobal(L, "LINEAR");
	lua_pushinteger(L, VAR_SINUSOIDAL); lua_setglobal(L, "SINUSOIDAL");
	lua_pushinteger(L, VAR_SPHERICAL); lua_setglobal(L, "SPHERICAL");
	lua_pushinteger(L, VAR_SWIRL); lua_setglobal(L, "SWIRL");
	lua_pushinteger(L, VAR_HORSESHOE); lua_setglobal(L, "HORSESHOE");
	lua_pushinteger(L, VAR_POLAR); lua_setglobal(L, "POLAR");
	lua_pushinteger(L, VAR_HANDKERCHIEF); lua_setglobal(L, "HANDKERCHIEF");
	lua_pushinteger(L, VAR_HEART); lua_setglobal(L, "HEART");
	lua_pushinteger(L, VAR_DISC); lua_setglobal(L, "DISC");
	lua_pushinteger(L, VAR_SPIRAL); lua_setglobal(L, "SPIRAL");
	lua_pushinteger(L, VAR_HYPERBOLIC); lua_setglobal(L, "HYPERBOLIC");
	lua_pushinteger(L, VAR_DIAMOND); lua_setglobal(L, "DIAMOND");
	lua_pushinteger(L, VAR_EX); lua_setglobal(L, "EX");
	lua_pushinteger(L, VAR_JULIA); lua_setglobal(L, "JULIA");
	lua_pushinteger(L, VAR_BENT); lua_setglobal(L, "BENT");
	lua_pushinteger(L, VAR_WAVES); lua_setglobal(L, "WAVES");
	lua_pushinteger(L, VAR_FISHEYE); lua_setglobal(L, "FISHEYE");
	lua_pushinteger(L, VAR_POPCORN); lua_setglobal(L, "POPCORN");
	lua_pushinteger(L, VAR_EXPONENTIAL); lua_setglobal(L, "EXPONENTIAL");
	lua_pushinteger(L, VAR_POWER); lua_setglobal(L, "POWER");
	lua_pushinteger(L, VAR_COSINE); lua_setglobal(L, "COSINE");
	lua_pushinteger(L, VAR_RINGS); lua_setglobal(L, "RINGS");
	lua_pushinteger(L, VAR_FAN); lua_setglobal(L, "FAN");
	lua_pushinteger(L, VAR_BLOB); lua_setglobal(L, "BLOB");
	lua_pushinteger(L, VAR_PDJ); lua_setglobal(L, "PDJ");
	lua_pushinteger(L, VAR_FAN2); lua_setglobal(L, "FAN2");
	lua_pushinteger(L, VAR_RINGS2); lua_setglobal(L, "RINGS2");
	lua_pushinteger(L, VAR_EYEFISH); lua_setglobal(L, "EYEFISH");
	lua_pushinteger(L, VAR_BUBBLE); lua_setglobal(L, "BUBBLE");
	lua_pushinteger(L, VAR_CYLINDER); lua_setglobal(L, "CYLINDER");
	lua_pushinteger(L, VAR_PERSPECTIVE); lua_setglobal(L, "PERSPECTIVE");
	lua_pushinteger(L, VAR_NOISE); lua_setglobal(L, "NOISE");
	lua_pushinteger(L, VAR_JULIAN); lua_setglobal(L, "JULIAN");
	lua_pushinteger(L, VAR_JULIASCOPE); lua_setglobal(L, "JULIASCOPE");
	lua_pushinteger(L, VAR_BLUR); lua_setglobal(L, "BLUR");
	lua_pushinteger(L, VAR_GAUSSIAN_BLUR); lua_setglobal(L, "GAUSSIAN_BLUR");
	lua_pushinteger(L, VAR_RADIAL_BLUR); lua_setglobal(L, "RADIAL_BLUR");
	lua_pushinteger(L, VAR_PIE); lua_setglobal(L, "PIE");
	lua_pushinteger(L, VAR_NGON); lua_setglobal(L, "NGON");
	lua_pushinteger(L, VAR_CURL); lua_setglobal(L, "CURL");
	lua_pushinteger(L, VAR_RECTANGLES); lua_setglobal(L, "RECTANGLES");
	lua_pushinteger(L, VAR_ARCH); lua_setglobal(L, "ARCH");
	lua_pushinteger(L, VAR_TANGENT); lua_setglobal(L, "TANGENT");
	lua_pushinteger(L, VAR_SQUARE); lua_setglobal(L, "SQUARE");
	lua_pushinteger(L, VAR_RAYS); lua_setglobal(L, "RAYS");
	lua_pushinteger(L, VAR_BLADE); lua_setglobal(L, "BLADE");
	lua_pushinteger(L, VAR_SECANT2); lua_setglobal(L, "SECANT");
	lua_pushinteger(L, VAR_TWINTRIAN); lua_setglobal(L, "TWINTRIAN");
	lua_pushinteger(L, VAR_CROSS); lua_setglobal(L, "CROSS");
	lua_pushinteger(L, VAR_DISC2); lua_setglobal(L, "DISC2");
	lua_pushinteger(L, VAR_SUPER_SHAPE); lua_setglobal(L, "SUPER_SHAPE");
	lua_pushinteger(L, VAR_FLOWER); lua_setglobal(L, "FLOWER");
	lua_pushinteger(L, VAR_CONIC); lua_setglobal(L, "CONIC");
	lua_pushinteger(L, VAR_PARABOLA); lua_setglobal(L, "PARABOLA");
	lua_pushinteger(L, VAR_SPLIT); lua_setglobal(L, "SPLIT");
	lua_pushinteger(L, VAR_MOVE); lua_setglobal(L, "MOVE");
	lua_pushinteger(L, -1); lua_setglobal(L, "RANDOM");
}
}
